==> mule-tooling-bootstrap-1.3.0-20220919-sources.jar: org/mule/tooling/client/bootstrap/internal/wrapper/CacheStorageMapWrapper.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.tooling.client.bootstrap.internal.wrapper;

import org.mule.tooling.client.api.cache.CacheStorage;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;

public class CacheStorageMapWrapper implements Map<String, Object> {

  private CacheStorage delegate;

  public CacheStorageMapWrapper(CacheStorage delegate) {
    this.delegate = delegate;
  }

  @Override
  public int size() {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean isEmpty() {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean containsKey(Object key) {
    return delegate.get((String) key).isPresent();
  }

  @Override
  public boolean containsValue(Object value) {
    throw new UnsupportedOperationException();
  }

  @Override
  public Object get(Object key) {
    return delegate.get((String) key).orElse(null);
  }

  @Override
  public Object put(String key, Object value) {
    return delegate.put(key, value);
  }

  @Override
  public Object remove(Object key) {
    throw new UnsupportedOperationException("Use the remove() and removeAll() methods from the keySet() to remove values from the cache");
  }

  @Override
  public void putAll(Map<? extends String, ?> m) {
    throw new UnsupportedOperationException();
  }

  @Override
  public void clear() {
    throw new UnsupportedOperationException();
  }

  @Override
  public Set<String> keySet() {
    return new MultipleKeysRemoverKeySet(delegate);
  }

  @Override
  public Collection<Object> values() {
    throw new UnsupportedOperationException();
  }

  @Override
  public Set<Entry<String, Object>> entrySet() {
    throw new UnsupportedOperationException();
  }

  private static class MultipleKeysRemoverKeySet implements Set<String> {

    private CacheStorage cache;
    private Set<String> originalKeySet;
    private Supplier<Set<String>> originalKeySetSupplier;

    MultipleKeysRemoverKeySet(CacheStorage cache) {
      this.cache = cache;
      this.originalKeySetSupplier = cache::getAllKeys;
    }

    //To avoid getting the key set for removing items.
    //A LazyValue could be used but it's not available.
    private Set<String> getOriginalKeySet() {
      if (originalKeySet == null) {
        originalKeySet = originalKeySetSupplier.get();
      }
      return originalKeySet;
    }

    @Override
    public int size() {
      return getOriginalKeySet().size();
    }

    @Override
    public boolean isEmpty() {
      return getOriginalKeySet().isEmpty();
    }

    @Override
    public boolean contains(Object o) {
      return getOriginalKeySet().contains(o);
    }

    @Override
    public Iterator<String> iterator() {
      return getOriginalKeySet().iterator();
    }

    @Override
    public Object[] toArray() {
      return getOriginalKeySet().toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
      return getOriginalKeySet().toArray(a);
    }

    @Override
    public boolean add(String s) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean remove(Object o) {
      cache.remove((String) o);
      return false;
    }

    @Override
    public boolean containsAll(Collection<?> c) {
      return getOriginalKeySet().containsAll(c);
    }

    @Override
    public boolean addAll(Collection<? extends String> c) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean retainAll(Collection<?> c) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean removeAll(Collection<?> c) {
      cache.remove(c.toArray(new String[] {}));
      return false;
    }

    @Override
    public void clear() {
      throw new UnsupportedOperationException();
    }
  }
}

==> mule-tooling-bootstrap-1.5.0-20220523-sources.jar: org/mule/tooling/client/bootstrap/internal/wrapper/CacheStorageMapWrapper.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.tooling.client.bootstrap.internal.wrapper;

import static java.util.Objects.requireNonNull;

import org.mule.tooling.client.api.cache.CacheStorage;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.function.Predicate;
import java.util.function.Supplier;

public class CacheStorageMapWrapper implements Map<String, Object> {

  private CacheStorage delegate;

  public CacheStorageMapWrapper(CacheStorage delegate) {
    requireNonNull(delegate, "Cache storage cannot be null");
    this.delegate = delegate;
  }

  @Override
  public int size() {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean isEmpty() {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean containsKey(Object key) {
    return delegate.get((String) key).isPresent();
  }

  @Override
  public boolean containsValue(Object value) {
    throw new UnsupportedOperationException();
  }

  @Override
  public Object get(Object key) {
    return delegate.get((String) key).orElse(null);
  }

  @Override
  public Object put(String key, Object value) {
    return delegate.put(key, value);
  }

  @Override
  public Object remove(Object key) {
    throw new UnsupportedOperationException("Use the removeIf() and removeAll() methods from the keySet() to remove values from the cache");
  }

  @Override
  public void putAll(Map<? extends String, ?> m) {
    throw new UnsupportedOperationException();
  }

  @Override
  public void clear() {
    throw new UnsupportedOperationException();
  }

  @Override
  public Set<String> keySet() {
    return new MultipleKeysRemoverKeySet(delegate);
  }

  @Override
  public Collection<Object> values() {
    throw new UnsupportedOperationException();
  }

  @Override
  public Set<Entry<String, Object>> entrySet() {
    throw new UnsupportedOperationException();
  }

  private static class MultipleKeysRemoverKeySet implements Set<String> {

    private CacheStorage cache;

    MultipleKeysRemoverKeySet(CacheStorage cache) {
      this.cache = cache;
    }

    @Override
    public int size() {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean isEmpty() {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean contains(Object o) {
      throw new UnsupportedOperationException();
    }

    @Override
    public Iterator<String> iterator() {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean removeIf(Predicate<? super String> filter) {
      cache.removeIf(filter);
      // We don't care if the key has been removed or not.
      return true;
    }

    @Override
    public Object[] toArray() {
      throw new UnsupportedOperationException();
    }

    @Override
    public <T> T[] toArray(T[] a) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean add(String s) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean remove(Object o) {
      cache.remove((String) o);
      return false;
    }

    @Override
    public boolean containsAll(Collection<?> c) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean addAll(Collection<? extends String> c) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean retainAll(Collection<?> c) {
      throw new UnsupportedOperationException();
    }

    @Override
    public boolean removeAll(Collection<?> c) {
      cache.remove(c.toArray(new String[] {}));
      return false;
    }

    @Override
    public void clear() {
      throw new UnsupportedOperationException();
    }
  }
}
